package alignment;

use strict;

#  takes: filename of an alignment in CLUSTALW format
#returns: ref to alignment hash, key = name, value = list of chars
#         ref to list of two hashes (namesref),
#         first  (NaNr): key = name (Na), value = number (Nr)
#         second (NrNa): key = number (Nr), value = name (Na)
#         while Nr refects the order of names in the alignment file     
sub read_clustal {
  my $file = shift;
#  print STDERR "file>$file<\n";
#  print STDERR "usage>$main::usage<\n";
#  print STDERR "gap>$main::gap<\n";
  my $gap=$main::gap;
  my %ali;
  my %NaNr;
  my %NrNa;
  my $index=0;
  open (CW, "<$file") or die "Can\'t open file \'$file\'! $main::usage";
  while(<CW>) {
    chomp;
    my @line=split;
    if (@line==2) {
      if ($line[1] =~ m/^[a-z$gap]*$/i) {
	unless (exists $NaNr{$line[0]}) {
	  $index++;
	  $NaNr{$line[0]}=$index;
	  #print "[>[>$line[0]<>$index<\n";
	  $NrNa{$index}=$line[0];
	}
	push @{$ali{$line[0]}}, split('',$line[1]);
      }
    }
  }
  close CW;
  my $alilen=0;
  my $passt=1;
#  print STDERR "alignment length ($file):\n";
  foreach my $org (keys(%ali)) {
#    print STDERR "org[$org]\n";
    if ($alilen==0) { $alilen=scalar(@{$ali{$org}}) }
    else {
      unless ($alilen == scalar(@{$ali{$org}})) {
	$passt=0;
      }
    }
#    printf STDERR "> %12s %8d\n", $org, scalar(@{$ali{$org}});
  }
  if ($passt) {
    return \%ali, [ \%NaNr, \%NrNa ];
  }
  else {
    die "Error at alignment! Use of characters other than [a-z], [A-Z]" .
	" and '$gap' for gaps?\n";
  }
}

#  takes: filename of an alignment in MULTIFASTA format
#returns: ref to alignment hash, key = name, value = list of chars
#         ref to list of two hashes (namesref),
#         first  (NaNr): key = name (Na), value = number (Nr)
#         second (NrNa): key = number (Nr), value = name (Na)
#         while Nr refects the order of names in the alignment file     
sub read_multifasta {
  my $file = shift;
  my %ali;
  my %NaNr;
  my %NrNa;
  my $gap=$main::gap;
  my $index=0;
  open (MFA, "<$file") or die "Can\'t open file \'$file\'! $main::usage";
 outer: while(<MFA>) {
   chomp;
   if ($_ =~ m/^>\s*(\S+)\s*/) {
     my $org = $1;
     unless (exists $NaNr{$org}) {
       $index++;
       $NaNr{$org}=$index;
       #print "[>[>$line[0]<>$index<\n";
       $NrNa{$index}=$org;
     }
#     @{$ali{$org}} = ();
     while(<MFA>) {
       chomp;
       if ($_ =~ m/^[a-z$gap]*$/i) {
	 push @{$ali{$org}}, split('',$_);
       }
       elsif ($_ =~ m/^>\s*(\S+)\s*/) {
	 redo outer;
       }
       elsif ($_ =~ m/^\s*$/) {
	 next;
       }
       else {
	 print STDERR "Warning: Something wrong in multifasta \'$file\':\n$_\n$main::usage";
	 last;
       }
     }
   }
 }
  close MFA;
  return \%ali, [ \%NaNr, \%NrNa ];
}

#  takes: ref to alignment hash
#returns: (1) ref to AtoS-hash:
#               key = name of sequence
#             value = array with Sequence-position at Alignment-position
#         (2) ref to StoA-hash:
#               key = name of sequence
#             value = array with Alignment-position at Sequence-position
#!!       gaps are indexed as x.5, where x is the pos of the previous letter
#!!       lc letters are treated the same way as UC letters
sub index_ali_1
{
  my $aliref=shift;
  my %AtoS; # Alignment to Sequence
  my %StoA; # Sequence to Alignment
  
  foreach my $seq (keys(%$aliref)) {
    my $Apos=0;         #position in the Alignment
    my $Spos=0;         #position in the Sequence
    my $befor=0.5;      #position of sequence between $befor-0.5 and $befor+0.5
    foreach my $char (@{$aliref->{$seq}}) {
      $Apos++;
      if ($char eq $main::gap) {
	${$AtoS{$seq}}[$Apos]=$befor; 
	next;
      }
      elsif ($char =~ m/[a-z]/i) { 
	$Spos++;
	$befor+=1;
	${$StoA{$seq}}[$Spos]=$Apos; 
      }
      else {
	die "Error at alignment! Use of characters other than [a-z], [A-Z]" .
	    " and '-' for gaps?\n";
      }
      ${$AtoS{$seq}}[$Apos]=$Spos; 
    }
  }
  return \%AtoS, \%StoA;
}

#  takes: ref to alignment hash
#returns: (1) ref to AtoS-hash:
#               key = name of sequence
#             value = array with Sequence-position at Alignment-position
#         (2) ref to StoA-hash:
#               key = name of sequence
#             value = array with Alignment-position at Sequence-position
#         gaps are indexed as x.y, where x  and y are the pos of
#!!            the previous and the following UC letter
#!!       lc letters are treated the same way as gaps
sub index_ali_2
{
  my $aliref=shift;
  my %AtoS; # Alignment to Sequence
  my %StoA; # Sequence to Alignment
  my @seqs = sort(keys(%$aliref));
  my $alilen= scalar(@{$aliref->{$seqs[0]}});  

  foreach my $seq (keys(%$aliref)) {
    my $Apos=0;         #position in the Alignment
    my $Spos=0;         #position in the Sequence
    my $prevUC=0;       #sequence position of previous UC letter
    my $prevApos=0;     #alignment position of the previous UC letter
    foreach my $char (@{$aliref->{$seq}}) {
      $Apos++;
      if ($char =~ m/[A-Z]/i) { 
	$Spos++;
	${$StoA{$seq}}[$Spos]=$Apos; 
	if ($char =~ m/[A-Z]/) { 
	  ${$AtoS{$seq}}[$Apos]=$Spos;
	  unless ($Apos == $prevApos+1) {
	    foreach my $posA ($prevApos+1..$Apos-1) {
	      my $posS = $prevUC . "," . $Spos;
	      ${$AtoS{$seq}}[$posA]=$posS;
	    }
	  }
	  $prevUC=$Spos;
	  $prevApos=$Apos;
	}
      }
      elsif ($char eq $main::gap) {next}
      else {
	die "Error at alignment! Use of characters other than [a-z], [A-Z]" .
	    " and '$main::gap' for gaps?\n";
      }
    }
    
    unless ($prevApos == $alilen) {
      foreach my $posA ($prevApos+1..$alilen) {
	my $posS = $prevUC . ",0";
	${$AtoS{$seq}}[$posA]=$posS;
      }
    }
      
  }
  return \%AtoS, \%StoA;
}


#com# use nt_frequency of window around position of interest
sub ali_entropy {
  my ($aliref, $AtoSref, $StoAref, $maxgaplen, $Ltotalmin, $window, $gapinAlpha)=@_;
  #my $window=23; #0 or odd number
  my @seqs = sort(keys(%$aliref));
  my $alilen= scalar(@{$aliref->{$seqs[0]}});
#  my $collen= scalar(keys(%$aliref));
  my $nt_freqref;
  unless ($window) {
    $nt_freqref=nt_frequency($aliref, [[1, $alilen]], \@main::Alpha);
  }
  my @entropy;
  for (my $i=0; $i<$alilen; $i++) {
    if ($window) {
      my $boundregref = boundary_window_1(0, $alilen-1, $i, $window);
      $nt_freqref=nt_frequency($aliref, $boundregref, \@main::Alpha);
    }
    my %iall;
    my $Ltotal=0; #total number of UC letters per column 
    my $total =0; #total number of symbols (UC letters, small gaps) per column 
    my $igapref = gap_length_2($i+1, $AtoSref, $StoAref);
    foreach my $seq (@seqs) { 
      if ($igapref->{$seq}->[0] eq "") {        #there is a UC letter, no gap
	if ($aliref->{$seq}->[$i] eq $main::gap) {
	  die "Exit - grosser Murx vor dem Herrn!\n";
	}
	else {
	  $iall{uc($aliref->{$seq}->[$i])}++;
	  $Ltotal++;
	}
      }
      elsif ($igapref->{$seq}->[0] == 1) {}        #5prime end gap (ignore)
      elsif ($igapref->{$seq}->[1] == -1) {}       #3prime end gap (ignore)
      elsif ($maxgaplen) {
	if ($igapref->{$seq}->[2] > $maxgaplen) {} #long gap (ignore)
 	else { $iall{$main::gap}++ }               #small gap
      }
      else { $iall{$main::gap}++ }                 #small gap
    }

    if ($main::debug) {
      print "[> Apos: " , $i+1 , "\n";
      # print_nt_freq(\%iall);
    }

    my $total=0;                                   #all symbols
    foreach my $s (keys(%iall)) {
      $total += $iall{$s};
    }
    if ($main::debug) {print " total: $total\nLtotal: $Ltotal\n"}

    #converting lc letters at this point, causes lc being treated as small gaps
    #foreach my $s (keys(%iall)) {                  #convert lc letters to gaps
    #  if ( $s =~ m/[a-z]/) {
    #	$iall{$main::gap} += $iall{$s};
    #	$iall{$s} = undef;
    #  }
    #}

    if ($main::debug) {print_nt_freq(\%iall)}

    foreach my $symbol (@main::Alpha) {
      my $tmp = 0;
      unless ($total <= 1 | $Ltotal < $Ltotalmin) { #if not something wrong...
	unless ($symbol eq $main::gap && $iall{$symbol} == $total) {
	  $tmp += $iall{$symbol}/$total;
	}
      }
      my $p = $nt_freqref->{$symbol};

      if ($tmp == 0) {
	$entropy[$i+1] += 0;
	if ($main::debug) {print " $tmp * log ($tmp / $p) "  , "\n"}
      }

      else {
	if ($main::debug) {print " $tmp * log ($tmp / $p) " , log $tmp/$p , "\n"}
	$entropy[$i+1] += $tmp*log $tmp/$p;
	#$entropy[$i+1] += ($tmp-$p)*log $tmp/$p;#joerg's Vorschlag
      }
    }
    if ($gapinAlpha == 0) {
      if ($iall{$main::gap} == 0) {
	$entropy[$i+1] += 0;
	if ($main::debug) {print " 0 * log (0) "  , "\n"}
      }
      else {
	if ($main::debug) {
	  print " $iall{$main::gap}/$total * log ($iall{$main::gap}/$total) " ,
	  log $iall{$main::gap}/$total , "\n";
	}
	$entropy[$i+1] += $iall{$main::gap}/$total *log $iall{$main::gap}/$total;
      }
    }
  }
  return \@entropy;
}

sub smooth_tropy {
  my($tropyref, $smooth)=@_;
  my $hs=$smooth/2-0.5;
  my @smotropy=@$tropyref[0..$hs];
  my $last=scalar(@$tropyref)-1;
  my $innerlast=$last;
  my @window=@$tropyref[1..$smooth];
  for (my $i=$hs+1; $i<$last-$hs; $i++) {
    my $sum;
    foreach my $e (@window) {
      $sum += $e;
    }
    $smotropy[$i]=$sum/$smooth;
    shift @window;
    push @window, $$tropyref[$i+$hs+1]; 
    $innerlast=$i+1;
  }
  push @smotropy, @$tropyref[$innerlast..$last];
  return \@smotropy;
}

sub print_tropy {
  my $tropyref = shift;
  for (my $Apos=1; $Apos<scalar(@$tropyref); $Apos++) {
    print "$Apos $tropyref->[$Apos]\n";
  }
  print "&\n";
}

#  takes: ref to alignment, Areg (alignment regions) and the Alphabeth
#returns: ref to hash, key = letter of Alphabeth, value = frequency
#!!       lc letters are treated as gaps
#!!       the returned frequences add up to one
sub nt_frequency {
  my ($aliref, $Areg, $Alpharef) = @_;
  my %rawfreq;
  foreach my $seq (keys(%$aliref)) {
    foreach my $reg (@$Areg) {
      foreach my $i (@{$aliref->{$seq}}[$reg->[0]-1..$reg->[1]-1]) {
	$rawfreq{$i}++;
      } 
    }
  }
  if ($main::debug) {print_nt_freq(\%rawfreq)};

  my %freq;
  my $sum;
  my $allgaps;
  foreach my $symbol (@$Alpharef) {
    foreach my $key (keys(%rawfreq)) {
#      if ($symbol =~ m/$key/i) {
      if ($symbol =~ m/$key/) {
	$freq{$symbol}+=$rawfreq{$key};
	$sum+=$rawfreq{$key};
      }
    }
  }
  foreach my $symbol (keys(%freq)) {  
      $freq{$symbol}=$freq{$symbol}/$sum;
  }
  return \%freq;
}

sub print_nt_freq {
  my $nt_freqref=shift;
  my $sum;
  foreach my $symbol (sort(keys(%$nt_freqref))) {
    print "$symbol <-> " , $nt_freqref->{$symbol} , "\n";
    $sum+=$nt_freqref->{$symbol};
  }
  print "---> sum: $sum\n\n"; 
}

sub gap_length_1 {
  my ($Apos, $AtoSref, $StoAref) = @_;
  my %Agap;
  foreach my $seq (keys(%$AtoSref)) {
    my $AposS = %$AtoSref->{$seq}->[$Apos];
    if ($AposS =~ m/\.5/) {
      $Agap{$seq}=[%$StoAref->{$seq}->[$AposS-0.5]+1,
		   %$StoAref->{$seq}->[$AposS+0.5]-1,
		   %$StoAref->{$seq}->[$AposS+0.5]-
		   %$StoAref->{$seq}->[$AposS-0.5]-1];
    }
    else {
      $Agap{$seq}=[ "", "", "0"];
    }
  }
  return \%Agap;
}

#  takes: an alignment position, AtoS and StoA indexes
#returns: ref to a hash, key = sequence name, value = three item list ->
#         [0]=first position of gap, [1]=last position of gap in alignment
#         [2]=gap length
sub gap_length_2 {
  my ($Apos, $AtoSref, $StoAref) = @_;
  my %Agap;
  foreach my $seq (sort(keys(%$AtoSref))) {
    my $AposS = %$AtoSref->{$seq}->[$Apos];
    #x,y while x is sequence position of upstream UC letter, y downstream
    if ($AposS =~ m/^(\d*),(\d*)$/) {
      $Agap{$seq}=[%$StoAref->{$seq}->[$1]+1,
		   %$StoAref->{$seq}->[$2]-1,
		   %$StoAref->{$seq}->[$2]-
		   %$StoAref->{$seq}->[$1]-1];
    }
    else {
      $Agap{$seq}=[ "", "", "0"];
    }
#not necessary:
#    if (${$Agap{$seq}}[1] == -1) {
#      my $alilen = scalar(@{$AtoSref->{$seq}})-1; 
#      ${$Agap{$seq}}[2] = $alilen-${$Agap{$seq}}[0]+1;
#    }
  }
  return \%Agap;
}

#  takes: boundaries of the sequence, beg and end
#         position i in sequence
#         window length (odd number)
#returns: sequence ranges (Areg like) to use for calculations
#!!       for positions i where the window would overhang the boundary
#         boundary_window_1 returns the window of        
sub boundary_window_1 {
  my ($beg, $end, $i, $window)= @_;
  my $boundregref;
  if    ($i < $beg+($window-1)/2) {      # left boundary
    $boundregref = [[$beg, $beg+($window-1)]];
  }
  elsif ($i > $end-($window-1)/2) {      #rigth boundary
    $boundregref = [[$end-($window-1), $end]];
  }
  else {
    $boundregref = [[$i-($window-1)/2,$i+($window-1)/2]];
  }
}

sub print_Agap {
  my $Agapref=shift;
  foreach my $seq (sort(keys(%$Agapref))) {
    printf "> %-10s\n", $seq;
    print "@{$Agapref->{$seq}}";
    print "\n";
  }
  print "\n\n";
  return;
}

sub print_order {
  my $namesref=shift;
  my @names = @$namesref; #NrNa
  
  print "\nNr <-> Na\n";
  foreach my $key (sort(keys(%{@names[1]}))) {
    print "$key <-> " . ${@names[1]}{$key} . "\n";
  }
  print "\n";
  
}

sub set_nr_for_names {
  my ($hashref, $namesref) = @_;
  my %retur;
#  foreach my $seq (@$regsref) {
  foreach my $seq (keys(%$hashref)) {
    my $Nr = $namesref->[0]->{$seq}; 
    $retur{$Nr} = $hashref->{$seq};
  }
  return \%retur;
}

sub get_namesref {
  my ($naref, $nrref) = @_;
  my %NrNa;
  my %NaNr;
  for (my $i=0; $i<scalar(@$naref); $i++) { 
    $NrNa{$nrref->[$i]}=$naref->[$i];
    $NaNr{$naref->[$i]}=$nrref->[$i];
  }
  return [ \%NaNr, \%NrNa ];
}

sub print_ali
{
  my ($aliref, $Aregref, $namesref) = @_;
  if (defined($namesref)) {
    foreach my $reg (@$Aregref) {
      foreach my $seq (keys(%$aliref)) {
	if ($namesref->[1]->{$seq} eq "") {
	  printf "%-10s ", $seq;
	}
	else {
	  printf "%-10s ", $namesref->[1]->{$seq};
	}
	print @{$aliref->{$seq}}[$reg->[0]-1..$reg->[1]-1];
	print "\n";
      }
      print "\n\n";
    }
  }
  return;
}

sub print_ali_mfa
{
  my ($aliref, $Aregref, $namesref) = @_;
  if (defined($namesref)) {
    foreach my $reg (@$Aregref) {
      foreach my $seq (keys(%$aliref)) {
	if ($namesref->[1]->{$seq} eq "") {
	  printf "> %-10s\n", $seq;
	}
	else {
	  printf "> %-10s\n", $namesref->[1]->{$seq};
	}
	print @{$aliref->{$seq}}[$reg->[0]-1..$reg->[1]-1];
	print "\n";
      }
      print "\n\n";
    }
  }
  return;
}


sub print_Areg {
    my $Aregref=shift;
    foreach my $ival (@$Aregref) {
	print "[ " . $ival->[0] . ", " . $ival->[1] . "]\n";
    }
}

sub print_index
{
  my ($AtoSref, $StoAref) = @_;
  foreach my $seq (sort(keys(%$AtoSref))) {
    print "$seq\n";
    print "Ali  Seq\n";
    for (my $A=1; $A<scalar(@{$AtoSref->{$seq}}); $A++) {
      print ">$A< >${$AtoSref->{$seq}}[$A]<\n";
    }
  }
  print "\n";
  foreach my $seq (sort(keys(%$StoAref))) {
    print "$seq\n";
    print "Seq  Ali\n";
    for (my $S=1; $S<scalar(@{$StoAref->{$seq}}); $S++) {
      print ">$S< >${$StoAref->{$seq}}[$S]<\n";
    }
  }
}

#xy=something calculated for each column in the alignment
#difflet= different letter per column
#!!!treat lc letters like UC letters
#!!!treat gaps as the 5th letter
#!!!treat all other symbols as missing data
sub xy_difflet
{
  my ($aliref) = @_;
  my @lAlpha = @main::Alpha;
#  push @lAlpha, $main::gap;  #!!!treat gaps as the 5th letter
  #print "lAlpha: @lAlpha\n";
  my @seqs = sort(keys(%$aliref));
  my $alilen= scalar(@{$aliref->{$seqs[0]}});
  my $seqs=scalar(@seqs);
  my @data;
  my @sumdata;
  my @sumsum;
  #print "SEQ-ORDER: @seqs\n";
  for (my $i=0; $i<$alilen; $i++) {
    my %iall;
    foreach my $seq (@seqs) { 
      $iall{uc($aliref->{$seq}->[$i])}++; #!!!treat lc letters like UC letters
    }
    my @colinfo;
    foreach my $lchar (@lAlpha) {
      if (exists $iall{uc($lchar)}) {     #!!!treat lc letters like UC letters
	$data[$i+1]++;
	push @colinfo, $iall{uc($lchar)};
      }
    }

    if (scalar(@colinfo) == 0)    { $sumdata[$i+1]=-1; #some mess
				  }
    elsif (scalar(@colinfo) == 1) { $sumdata[$i+1]= 0; #conserved column
				    $sumsum[0]++;
				  }
    elsif (scalar(@colinfo) == 2)
    {
      if ($colinfo[0] == 1 |
	  $colinfo[1] == 1)       { $sumdata[$i+1]= 1; #singelton
				    $sumsum[1]++;
				  }
      else                        { $sumdata[$i+1]= 2; #parsimony informative
				    $sumsum[2]++;
				  }
    }
    else                          { $sumdata[$i+1]= 2; #parsimony informative
				    $sumsum[2]++;
				  }
  }
  print "    len    cons     var prasinf  single\n";
  printf "%7d %7d %7d %7d %7d\n", $alilen, $sumsum[0], $sumsum[1]+$sumsum[2], $sumsum[2], $sumsum[1];

  return \@data, \@sumsum;
}

#xy=something calculated for each column in the alignment
#splits=splits per cyclic(!) order of sequences
sub xy_splits
{
  my ($aliref, $namesref) = @_;
  #get sequence names in the same order as in the input file
  my @seqs;
  foreach my $Nr (sort(keys(%{$namesref->[1]}))) {
    push @seqs, $namesref->[1]->{$Nr};
  }
  #print "order: @seqs\n";
  print "gap=>$main::gap<\n";
  my $alilen= scalar(@{$aliref->{$seqs[0]}});
  my @data;

  for (my $i=0; $i<$alilen; $i++) {
    my $lastchar;
    foreach my $seq (@seqs) { 
      unless (defined($lastchar)) {
	$lastchar=uc($aliref->{$seq}->[$i]);
	$data[$i+1]=0;
	next;
      }

      unless ($lastchar eq uc($aliref->{$seq}->[$i])) {
	if ($lastchar eq $main::gap |
	    uc($aliref->{$seq}->[$i]) eq $main::gap) {
	  # mach nix
	}
	else {
	  $data[$i+1]++;
	  $lastchar=uc($aliref->{$seq}->[$i]);
	}

      }
    }
    unless ($lastchar eq uc($aliref->{$seqs[0]}->[$i])) {
      if ($lastchar eq $main::gap |
	  uc($aliref->{$seqs[0]}->[$i]) eq $main::gap) {
	# mach nix
      }
      else {
	$data[$i+1]++;
      }
      $lastchar=uc($aliref->{$seqs[0]}->[$i]);
    }
  }
  return \@data;
}

#reorder columns of an alignment
#picks $number random columns from the alignment
sub rand3 {
  my ($aliref, $seqsref, $number)=@_;
  my @seqs=@$seqsref;
  my $alilen= scalar(@{$aliref->{$seqs[0]}});
  my $len=$alilen;
  my @poses;
  if (defined ($number)) {
    if ($number <= $len) {} #everything is fine
    else {$number=$len; print STDERR "number out of alignment range!\n";}
  }
  else {$number=$len} 
  
  foreach my $i (0..$alilen-1) {
    $poses[$i]=$i;
  }
  my @list;
  do {
    my $r = int(rand($len));
    my $e = splice(@poses,  $r, 1);
    push @list, $e;
    $len--;
    $number--;
  } until ($number <= 0);

  #print "list (" . scalar(@list) . "): @list\n";

  my %randali;
  for (my $i=0; $i<scalar(@list); $i++) {
    foreach my $seq (keys(%$aliref)) {
      ${$randali{$seq}}[$i]=${$aliref->{$seq}}[$list[$i]];
    }
  }
  return \%randali;
}

#replace letters in the alignment by other randomly chosen letters
#using a given nt_frequency, ntrand (usually taken from the original alignment)
sub rand2 {
  my ($aliref, $Aregref, $seqsref, $window)=@_;
  my @seqs=@$seqsref;
  my $alilen= scalar(@{$aliref->{$seqs[0]}});
  my %ntrand;
  my %randali;
  unless ($window) {
    my $nt_freqref=nt_frequency($aliref, [[1, $alilen]], \@main::Alpha);
    #this is to define intervalls on the intervall [0,1[
    #for each letter of Alpha. If the random number lies into the intervall
    #of a certain letter, this letter is chosen. 
    my $p=0;
    foreach my $A (@main::Alpha) {
      $p+=$nt_freqref->{$A};
      $ntrand{$A}=$p;
    }
    $ntrand{$main::Alpha[$#main::Alpha]}=1;
  }

  foreach my $reg (@$Aregref) {                      #for each Areg 
    for (my $i=$reg->[0]-1; $i<=$reg->[1]-1; $i++) { #for each column in Areg 
      if ($window) {
	my $boundregref = boundary_window_1(0, $alilen-1, $i+1, $window);
	my $nt_freqref=nt_frequency($aliref, $boundregref, \@main::Alpha);
	my $p=0;
	foreach my $A (@main::Alpha) {
	  $p+=$nt_freqref->{$A};
	  $ntrand{$A}=$p;
	}
	$ntrand{$main::Alpha[$#main::Alpha]}=1;
      }
      foreach my $seq (keys(%$aliref)) {
	my $letter=$main::gap;
	if  (${$aliref->{$seq}}[$i] eq $main::gap) { #if there is a gap
	  push @{$randali{$seq}}, $letter;
	  next;
	}
	else {                                       #if there is a letter
	  my $random=rand(1);
	  #chose a letter with probability equal to nt_freq
	  foreach my $L (@main::Alpha) {
	    if ($random < $ntrand{$L}) {
	      $letter=$L;
	      last;
	    }
	  }
	  #deside whether letter has to be lc or uc 
	  if (${$aliref->{$seq}}[$i] =~ m/^[a-z]$/) {    #original letter is lc
	    $letter=lc($letter);
	  }
	  elsif (${$aliref->{$seq}}[$i] =~ m/^[A-Z]$/) { #original letter is UC
	    $letter=uc($letter);
	  }
	  else {
	    die "What letter ist that \'$letter\'?!\n";
	  }
	  push @{$randali{$seq}}, $letter;
	}
      }
    }
  }
  return \%randali;
}

#old method:
sub rand1 {
  my ($seqs, $alilen, $seqlenref)=@_;
  my %ali;
  my %gaps;
  push my @nucleotide, @main::Alpha, $main::gap;
  #build a random alignment with given length alilen
  for (my $seq=1; $seq<=$seqs; $seq++) {
    my @strand;
    for (my $i=0; $i<= $alilen-1; $i+=1) { # build a sequence
      my $random = (rand 5)%5;
      $strand[$i] = $nucleotide[$random];
      if ($nucleotide[$random] eq $main::gap) {
	$gaps{$seq}++;
      }
    }
    $ali{$seq}=\@strand;
  }
  
  #adjust the random alignment 
  #so that sequences have certain length
  #attention: columns with all gap characters may occure!
  foreach my $seq (keys(%$seqlenref)) {
    if ($alilen-$gaps{$seq} == $seqlenref->{$seq}) {next;}
    elsif ($alilen-$gaps{$seq} > $seqlenref->{$seq})  {
      # replace letters with gaps:
      for (my $r=1; $r<=$alilen-$gaps{$seq}-$seqlenref->{$seq}; $r++) {
	my $random = (rand $alilen) % $alilen; #draw a random position
	if (${$ali{$seq}}[$random] eq $main::gap) {
	  redo;
	}
	else {
	  ${$ali{$seq}}[$random] = $main::gap; # replace letter by gap
	}
      }
    }
    else { 
      # replace gaps with letters:
      for (my $r=1; $r<=$seqlenref->{$seq}-($alilen-$gaps{$seq}); $r++) {
	my $random = (rand $alilen) % $alilen; #draw a random position
	if (${$ali{$seq}}[$random] eq $main::gap) {
	  my $nr = (rand 4)%4;
	  ${$ali{$seq}}[$random] = $nucleotide[$nr]; # relpace gap by letter
	}
	else {
	  redo;
	}
      }
    }
  }
  return \%ali;

}

#multiplies the values of two xy-data series
sub multiply {
  my ($xy1ref, $xy2ref) = @_;
  my @multi;

  if (ref($xy2ref) eq 'ARRAY') { 
    if (scalar(@$xy1ref)==scalar(@$xy2ref)) {
      for (my $Apos=1; $Apos<scalar(@$xy1ref); $Apos++) {
	$multi[$Apos]=($xy1ref->[$Apos])*($xy2ref->[$Apos]);
      }
    }
    else { die "xy-data series are not the same length!\n" }
  }
  else {
    for (my $Apos=1; $Apos<scalar(@$xy1ref); $Apos++) {
      $multi[$Apos]=($xy1ref->[$Apos])*($xy2ref);
    }
  }
  return \@multi;
}

#divides the values of xy-data serie 1 by xy-data serie 2
sub devide {
  my ($xy1ref, $xy2ref) = @_;
  die "xy-data series are not the same length!\n" 
      unless scalar(@$xy1ref)==scalar(@$xy2ref);
  my @div;
  for (my $Apos=1; $Apos<scalar(@$xy1ref); $Apos++) {
    if ($xy2ref->[$Apos] == 0) {
      $div[$Apos]='n.a.';
    }
    else {
      $div[$Apos]=($xy1ref->[$Apos])/($xy2ref->[$Apos]);
    }
  }
  return \@div;
}
1;
