package exons;

use lib "/homes/gintonic/sonja/bin/UTILS/Modules";
use strict;
use interval;
use footprint;

#  takes: filename of PSFILE and the type (if different from 'GENES')
#returns: a hash (exons),
#         key = sequence name
#         value = list of exons (a three item list):
#         [0]=begin, [1]=end, [2]=exon
sub read_exons {
  my ($psfile, $type) = @_;
  open PS, "< $psfile" or die "Can't open >$psfile< !\n$main::usage";;
  my $lin=0;
  my $auf=0;
  my %exons;
  my $seq;

  unless (defined($type) & $type ne '' & $type ne '0') {
    $type = 'GENES';
  }
  
  while (<PS>) {
    $lin++;
    my $line = $_;
    if ($line =~ m/^\/$type/) {
      $auf = _k_level($auf, $line);
    }
    elsif ($auf > 0) {
      if ($line =~ m/^(.*)\s+(\d+)\s+(\d+)\s+\((\w+)\)\s*(.*)$/ ) {
	$auf = _k_level($auf, $1); #davor
	if ($auf == 2) {
	  $seq=$4;
	}
	$auf = _k_level($auf, $5); #danach
      }
      elsif ($line =~ m/^(.*)\s+(\d+)\s+(\d+)\s+\(\s*([\w-]*)\s*\)\s*(.*)$/) {
	$auf = _k_level($auf, $1);	  
	if ($auf == 4 ) {
	  push @{$exons{$seq}}, [ $2, $3, $4 ];
	}
	$auf = _k_level($auf, $5);
      }
      else {
	$auf = _k_level($auf, $line);
      }
    }
  }
  return \%exons;
}

#  takes: Ref to exons hash (as returnd by read_exons)
#returns: a hash (igrs),
#         key = sequence name
#         value = list of igrs (a three item list):
#         [0]=begin, [1]=end, [2]=igr_name
sub IGRs {
  my $exonref=shift;
  my %igrs;
  foreach my $seq (keys(%$exonref)) {
    my $igrsref=footprint::invert_Areg2(\@{$exonref->{$seq}});
    $igrs{$seq}=$igrsref;
  }
  return \%igrs;
}

#  takes: Ref to exons hash (as returnd by read_exons)
#         Ref to order (as returned by read_orderfile)
#returns: a hash (igrs),
#         key = sequence name
#         value = list of igrs (a three item list):
#         [0]=gene at begin, [1]=gene at end, [2]=igr_length
#         the newly introduced names of 5prime and 3prime ends
sub IGRs_2 {
  my $exonref=shift;
  my %igrs;
  my $b='5prime';
  my $e='3prime';
  foreach my $seq (keys(%$exonref)) {
    my $beg=$b;
    my $begpos='0';
    my $end=$e;
    my $endpos;
    foreach my $gene (@{$exonref->{$seq}}) {
      #gene[0]=begin of gene, gene[1]=end of gene, gene[2]=name of gene
      push @{$igrs{$seq}}, [$beg, $gene->[2], $gene->[0]-$begpos-1];
      $beg=$gene->[2];
      $begpos=$gene->[1];
    }
    if ($endpos) {
      push @{$igrs{$seq}}, [$beg, $end, $endpos-$begpos-1];
    }
  }
  return \%igrs, $b, $e;
}

#  takes: Ref to igrs hash (as returned by IGRs)
#   does: calculates the absolut IGR_length
#         and prints it in xmgrace-like format
#returns; nothing 
sub print_IGR_length {
  my ($igrsref)=shift;
  foreach my $seq (keys(%$igrsref)){
    print "#---\n#$seq\n#---\n";
    foreach my $igr (@{$igrsref->{$seq}}){
      print "##".$igr->[2]."\n";
      print "".($igr->[1]-$igr->[0]+1)."\n";
    }
  }
}

sub print_IGRs_tex {
  my @alldata=@_;

  foreach my $seq (keys(%{$alldata[0]})) {
    foreach my $data (@alldata) {
      my @a=@{$data->{$seq}};
      print "\\begin{tabular}{|l||";
      print "r|" for 0..$#a;
      print "}\n\\hline\\hline\n"."               ";
      foreach my $e (@a) {
	my $colhead=$e->[0]."-".$e->[1];
	printf " & %15s", $colhead;
      }
      printf "\\\\\n%15s", $seq;
      foreach my $e (@a) {
	if (int($e->[2]) eq $e->[2]) { #integers
	  printf " & %15d", $e->[2];
	}
	else {                         #floating point numbers
	  printf " &      %9.3f%%", 100*$e->[2];
	}
      }
      print "\\\\\n\\hline\n";
      print "\\end{tabular}\n\n";
    }
    print "\\hline\n";
  }
}
      


#privat method called by sub read_exons
sub _k_level {
  my ($auf, $string) = @_;
  foreach my $l (split('',$string)) {
    if ($l eq '[') { $auf++ }
    elsif ($l eq ']') { $auf-- }
  }
  return $auf;
}

sub exons2Areg {
    my ($StoAref, $exonsref)=@_;
    my @Areg;
    foreach my $seq (keys(%$exonsref)) {
	foreach my $gene (@{$exonsref->{$seq}}) {
	   my @tmp = (%$StoAref->{$seq}->[$gene->[0]],
		      %$StoAref->{$seq}->[$gene->[1]]);
	   push @Areg, [@tmp];
       }
    }
    my $minAregref=interval::intervaller(\@Areg);
    #print "\n---\n\nMINAREG:\n";
    #print_Areg($minAregref);
    return $minAregref;
}

sub print_exons {
  my ($exonsref)=@_;
  #print "\n[>[>[> Exons:\n";
  foreach my $seq (keys(%$exonsref)) {
    print "$seq\n  [\n";
    foreach my $AEN (@{$exonsref->{$seq}}) {
      print "    [ @$AEN ]\n";
    }
    print "  ]\n";
  }
}

sub print_Areg {
    my $Aregref=shift;
    foreach my $ival (@$Aregref) {
	print "[ " . $ival->[0] . ", " . $ival->[1] . "]\n";
    }
}

#takes   orderfile
#returns Ref to hash,
#        key = sequence name
#        value = two hashes
#                hash[0]: key = general gene name, value = individual gene name
#                hash[1]: key = individual gene name, value = general gene name
#        Ref to an array, giving the order of general gene names
sub read_orderfile {
  my ($file)=shift;
  my $linelength;
  my @order;
  my %order;
  my @o;
  my $genes=0;

  open(ORDER, "<$file") or die "Can\'t open \'$file\'!\n";

  while(<ORDER>) {
    my @line=split;
    if ($. == 1) {
      @order = map {[$_]} @line;
      $linelength = @order; next;
    }
    die "wrong number of elements at line \'$.\' of \'$file\'!\n"
	if @line != $linelength;
    $genes++;
    push @o, $line[0];
    map { push @{$order[$_]}, $line[$_] } 0..$#order;
  }

  close ORDER;
  my $seqs=$linelength-1;

  for (my $i=1; $i<=$seqs; $i++)    { #for each organism
    my (%hash1,%hash2);
    for (my $j=1; $j<=$genes; $j++) { #for each gene
      $hash1{$order[0][$j]}=$order[$i][$j];
      $hash2{$order[$i][$j]}=$order[0][$j];
    }
    $order{$order[$i][0]}=[\%hash1,\%hash2];
  }

  return \%order, \@o; 
}

sub print_order {
  my ($orderref, $oref)=@_;
  my $it='1';
  foreach my $seq (keys(%$orderref)) {
    print "$seq [\n";
    foreach my $gene (keys(%{${$orderref->{$seq}}[$it]})){
      print "  $gene -> ". ${${$orderref->{$seq}}[$it]}{$gene} . "\n";
    }
    print "]\n";
  }
  print "order: @$oref\n";
}

#  takes: 1) Ref to hash of IGRs (as returned from IGR_2(!)) or similar
#         2) Ref to hash of IGRs (as returned from IGR_2(!)) or similar
#            (at least holding data for reference sequence)
#         3) Ref to array holding the common gene names in their genomic order 
#         4) sequence name of the reference sequence
#returns: Ref to hash of IGRs with relativ values as IGR property
sub rel_IGR_length {
  my ($iref, $jref, $oref, $norm)=@_;

  my %religrs;

  my %o;
  my %r;
  my %r3gen; #key=gene in ref. seq., value=next gene in ref. seq.
  my %tmp;
  #write genes existing in the reference sequence into an hash (tmp)
  foreach my $igr (@{$jref->{$norm}}) {
    $tmp{$igr->[0]}++;
    $tmp{$igr->[1]}++;
  }
  #assign those genes their place in array order (r -> hash)
  #assign all genes their place in array order (o -> hash)
  my $lastexist='';
  for (my $i=0; $i<=scalar(@$oref); $i++) {
    if (exists $tmp{$oref->[$i]}){
      $r{$oref->[$i]}=$i;
      $r3gen{$lastexist}=$oref->[$i];
      $lastexist=$oref->[$i];
    }
    $o{$oref->[$i]}=$i;
  }

  foreach my $seq (keys(%$iref)){
    foreach my $igr (@{$iref->{$seq}}) {
      if ($main::debug) {print "---\na  seq: @$igr\n";}
      if (exists($r{$igr->[0]})) {
	if (exists($r{$igr->[1]})) {
	  if ($main::debug) {print "        matching...\n";}
	  my $sum=0;
	  my $in=0;
	  foreach my $refigr (@{$jref->{$norm}}) {
	    if ($refigr->[0] eq $igr->[0]) {
	      if ($main::debug) {print "refseq: start\n";}
	      $in=1;
	    }
	    if ($in) {
	      if ($main::debug) {print "refseq: igr: @$refigr\n";}
	      $sum+=$refigr->[2];
	    }
	    if ($refigr->[1] eq $igr->[1]) {
	      if ($main::debug) {print "refseq: finish\n";}
	      my $quot=$igr->[2]/$sum;
	      push @{$religrs{$seq}}, [$igr->[0], $igr->[1], $quot];
	      last;
	    }
	  }
	}
	else {
	  #the next gene in norm:
	  $r3gen{$igr->[0]};
	  # if the next gene in norm is present in the current sequence
          # do the revers of above
          # else relative IGR length can't be calculated
          # NA-1 means: not applicable but CAN probably be calculated!
	  push @{$religrs{$seq}}, [$igr->[0], $igr->[1], 'NA-1'];
	}
      }
      else { #can't calculate relative IGR length
	push @{$religrs{$seq}}, [$igr->[0], $igr->[1], 'NA-2'];
      }
    }
  }
  return \%religrs;
}  
  


1;
