#!/usr/bin/perl
#  program: mAAna_tropy.pl
# language: perl, v5.6.1 built for i386-linux
#   author: Sonja Prohaska
#institute: Lehrstuhl fuer Bioinformatik, Institut fuer Informatik
#   adress: Haertlstr. 16-18, 04107 Leipzig
#   e-mail: sonja@bioinf.uni-leipzig.de
#     date: 17.08.2004

#not aplicable to alignments with 47*2.500.000nt! 

###############################################################################
# adopt the following line to your needs
# (and save mAAna_tropy.pl as *plain* txt file again):
use lib "~/bin/Modules";
###############################################################################

use strict;
use alignment;
use footprint;
use exons;
use vars qw/$usage $gap @Alpha $debug/;

$usage= << "JUS";
  usage: mAAna_tropy.pl [OPTIONS] CLUSTALW-ALIGNMENTS
  needs: linux operating system
         Modules/alignment.pm
         Modules/footpint.pm
         Modules/exons.pm
         Modules/interval.pm
         (set "lib" to the location of the "Modules" directory
          in line 15 of mAAna_tropy.pl)
options: -h|-help get this usage message
         -Alpha=  default: ACGT (case insensitive)
         -gap=    default: -
         -maxgap= length of stretches of gaps that should be treated as end gaps
         -minLet= minimum number of letters per column to do calculations, default: 2
         -smooth= (odd) smoothing, default: 1 (no smoothing)
         -cut=    (floating point number) select alignment fragments
                  with higher information content
         -minlen= minimal length of alignment fragments, default: 6
         -dat=    xmgrace: write xmgrace data file to STDOUT 
                      tex: write tracker-like LaTeX table to STDOUT
                           (only together with option -cut)
                     html: write tracker-like HTML table to STDOUT
                           (only together with option -cut)
         -motifs  write motifs as multifasta to file: motifs.mfa
                  (only together with option -cut)
         -conc    write cocatenated fragments to fasta file: concali.fa
                  (only together with option -cut)
         -exons=  PS-FILE, exclude fragments that result from aligned exons 
         -window= length of window to calculate nt_frequence (default: alilen)
         -boot=   number of bootstraps (random alignments to generate)
                  default: 0
JUS

my @options = grep {substr($_, 0, 1) eq '-'} @ARGV;
my @others  = grep {substr($_, 0, 1) =~ /^[^-]/} @ARGV;

die "$usage\n" unless (scalar(@others)>=1);
my $ali_file = shift(@others);
$debug = 0;
my $replacelc = 1; #replace lc letters by gaps
$gap = '-';
@Alpha = ('A', 'C', 'G', 'T');
my $gapinAlpha = 0;
my $maxgaplen = 0; #equivalent to undefined
my $Ltotalmin = 2; #number of letters per column to alow calculation
                   #otherwise calculation = 0
my $smooth = 1;
my $cut = 0;
my $minlen = 6;
my $dat = 0;        #xmgrace output
my $motifs = 0;
my $conc = 0;
my $file_exons = 0;
my $window = 0;
my $boot = 0;
my $string = join (' ', @options, $ali_file);

#setting parameters:
foreach my $option (@options){
  if    ($option eq '-d')               { $debug = 1  }  
  elsif ($option eq '-h' )              { print $usage; exit}
  elsif ($option eq '-help')            { print $usage; exit}
  elsif ($option =~ m/^-gap=(.)$/)      {   $gap = $1 }
  elsif ($option =~ m/^-Alpha=(.*)$/)   { @Alpha = split(//, $1) }
  elsif ($option =~ m/^-maxgap=(\d*)$/) { $maxgaplen = $1 }
  elsif ($option =~ m/^-minLet=(\d*)$/) { $Ltotalmin = $1 }
  elsif ($option =~ m/^-smooth=(\d*)$/) { $smooth = $1 }
  elsif ($option =~ m/^-cut=(\d+.\d+)$/){ $cut = $1 }
  elsif ($option =~ m/^-minlen=(\d+)$/) { $minlen = $1 }
  elsif ($option =~ m/^-dat=xmgrace$/)  { $dat = 1  }
  elsif ($option =~ m/^-dat=tex$/)      { $dat = 2  } 
  elsif ($option =~ m/^-dat=html$/)     { $dat = 3  } 
  elsif ($option =~ m/^-motifs$/)       { $motifs = 1  }
  elsif ($option =~ m/^-conc$/)         { $conc = 1  }
  elsif ($option =~ m/^-exons=(.*)$/) { $file_exons = $1 ; }
  elsif ($option =~ m/^-window=(\d*)$/) { $window = $1 }
  elsif ($option =~ m/^-boot=(\d*)$/)     { $boot   = $1; }
  else  { die "Invalid input: \'$option\'!\n$usage"; }
}

#if gap element of Alphabet
foreach my $symbol (@Alpha) {
  if ($gap eq $symbol) {
    $gapinAlpha =1;
    last;
  }
}

my ($aliref, $namesref)= alignment::read_clustal($ali_file);
my @seqs = sort(keys(%$aliref));
my $alilen= scalar(@{$aliref->{$seqs[0]}});
#if ($debug) {print "alignment:\n";}
#if ($debug) {alignment::print_ali($aliref, [[1, $alilen]], $namesref);}
my ($AtoSref, $StoAref)=alignment::index_ali_2($aliref);
#if ($debug) {print "alignment index:\n";}
#if ($debug) {alignment::print_index($AtoSref, $StoAref);}
if ($debug) {print "order:\n";}
if ($debug) {alignment::print_order($namesref);}
print STDERR " Alpha: @Alpha\n";
print STDERR "   gap: $gap\n";
print STDERR "maxgap: $maxgaplen\n";
print STDERR "smooth: $smooth\n";
print STDERR "   cut: $cut\n\n";
#my $nt_freqref=alignment::nt_frequency($aliref, [[1, $alilen]], \@Alpha);
#if ($debug) {print "nt_frequency:\n"}
#if ($debug) {alignment::print_nt_freq($nt_freqref)}
my $tropyref = alignment::ali_entropy($aliref, $AtoSref, $StoAref, $maxgaplen, $Ltotalmin, $window, $gapinAlpha);
#alignment::print_tropy($tropyref);

my $smotropyref=alignment::smooth_tropy($tropyref, $smooth);
if ($dat==1) {
  print "#mAAna_tropy maxgap=$maxgaplen, smooth=$smooth, cut=$cut; tropy\n";
  alignment::print_tropy($smotropyref);
}

if ($boot) {
  my @boottropy;
  for (my $i=1; $i<=$boot; $i++) {
    my $randaliref=alignment::rand2($aliref, [[1, $alilen]], \@seqs, $window);
    my $randtropyref = alignment::ali_entropy($randaliref, $AtoSref, $StoAref, $maxgaplen, $Ltotalmin, $window, $gapinAlpha);
    $randtropyref = alignment::smooth_tropy($randtropyref, $smooth);
    for (my $Apos=1; $Apos<scalar(@$randtropyref); $Apos++) {
      $boottropy[$Apos] += 1/$boot*($randtropyref->[$Apos]);
    }
  }
  #substract noise from bootstrapping from smotropyref
  for (my $Apos=1; $Apos<scalar(@boottropy); $Apos++) {
    $smotropyref->[$Apos] -= $boottropy[$Apos];
  }

  if ($dat==1) {
    print "#mAAna_tropy maxgap=$maxgaplen, smooth=$smooth, cut=$cut, boot=$boot;  difference between 'tropy' and 'tropy' of bootstrapped alignment\n";
    alignment::print_tropy($smotropyref);
  }
}

if ($cut) {
  my $alifragref=footprint::fragmentali($smotropyref, $cut, $alilen);
#  print "vorher:\n";
#  footprint::print_alifrag($alifragref);
  my $selalifragref=footprint::select_alifrag($alifragref, $minlen);
  #footprint::print_alifrag($selalifragref);
#  print "nachher:\n";

  if ($file_exons) {
      my $exonsref = exons::read_exons($file_exons, 'GENES');
      #$exonsref = alignment::set_nr_for_names($exonsref, $namesref);
      #exons::print_exons($exonsref);
      $selalifragref=footprint::select_alifrag2($selalifragref, $StoAref, $exonsref);
      #footprint::print_alifrag($selalifragref);
  }
  if ($dat==2) {
    print "%mAAna_tropy $string\n"; 
    footprint::print_tex2($aliref, $selalifragref, $namesref, $AtoSref, $StoAref, $minlen, $string);
  }
  elsif ($dat==3) {
    print "<!--mAAna_tropy $string-->\n"; 
    footprint::print_html($aliref, $selalifragref, $namesref, $AtoSref, $StoAref, $minlen, $string);
  }
  elsif ($dat==0) {
    print "#mAAna_tropy $string\n";
    footprint::print_alifrag($selalifragref);
    alignment::print_ali($aliref, $selalifragref, $namesref);
  }
  my $L_distref=footprint::frag_L_dist($alifragref);
  if ($debug) {footprint::print_L_dist($L_distref)}
  if ($motifs) {
      footprint::print_Areg2multifasta2($aliref, $selalifragref, $AtoSref, $StoAref, $namesref, $minlen);
      #footprint::print_Areg2bbqin($aliref, $selalifragref, $AtoSref, $namesref, $minlen);
  }
  if ($conc) {
      footprint::print_concali2($aliref, $AtoSref, $StoAref, $selalifragref, $minlen, $string);
  }
}

#my %newali;
#my %multipl;
#for (my $i=0; $i<$alilen; $i++) { #$i+1=$Apos
#  my $nrofgaps=0;
#  foreach my $seq (@seqs) {
#    if (${$aliref->{$seq}}[$i] eq $gap) {
#      $nrofgaps++;
#    }
#  }
#  $multipl{$nrofgaps}++;
#  unless ($nrofgaps == scalar(@seqs)) {
#    foreach my $seq (@seqs) {
#      push @{$newali{$seq}}, ${$aliref->{$seq}}[$i];
#    }
#  }
#}
#my $newalilen= scalar(@{$newali{$seqs[0]}});
#if ($debug) {print "new:\n";}
#print_ali(\%newali, [[1, $newalilen]]);


